clear; clc; close all; 
plotops
gray = 0.6*ones(1,3);

% Save
save_fig = 0;
pres_save = 0;

% Compute bounding logical
limit = 3;
interval = 0.01;
b011_grid = [0.5,1,1.5];
b022_grid = [0.5,1,1.5];
b012_grid = -limit:interval:limit;
b021_grid = -limit:0.02:limit;
[b011_gr,b022_gr,b012_gr,b021_gr] = ndgrid(b011_grid,b022_grid,b012_grid,b021_grid);
nodes = numel(b011_gr);
griddim = size(b011_gr);
b011_pts = numel(b011_grid);
b022_pts = numel(b022_grid);
cond = zeros(griddim);
for inode = 1:nodes
    b011 = b011_gr(inode);
    b022 = b022_gr(inode);
    b012 = b012_gr(inode);
    b021 = b021_gr(inode);
    c2 = (b011*b021+b012*b022)/sqrt(b011^2+b012^2);
    if b021<0
        cond(inode) = c2<=b021;
    elseif b021>0
        cond(inode) = c2>=b021;
    else
        cond(inode) = 1;
    end
end

% Plot bounding region (b011=1)
figpos = [1,5,5,5];
axispos = [.1,.1,.86,.87];
figure; hold on; box on;
set(gcf,'Position',figpos,...
        'PaperPosition',[0,0,figpos(3:4)],...
        'PaperSize',figpos(3:4));
set(gca,'Position',axispos);
b011idx = find(b011_grid==1);
b022idx = find(b022_grid==1);
cond_tmp = cond(b011idx,b022idx,:,:);
b012_gr_tmp = b012_gr(b011idx,b022idx,:,:);
b021_gr_tmp = b021_gr(b011idx,b022idx,:,:);
xaxis = b012_gr_tmp(cond_tmp==1);
yaxis = b021_gr_tmp(cond_tmp==1);
scatter(xaxis(:),yaxis(:),2,'.',...
    'MarkerFaceColor',gray,'MarkerEdgeColor',gray)
xlabel('Response of Uncertainty to Level Shock ($b_0^{12}$)','interpreter','latex')
ylabel('Response of Output to Uncertainty Shock ($b_0^{21}$)','interpreter','latex')
yline(0,'LineWidth',1);
xline(0,'LineWidth',1);
set(gca, 'layer', 'top');
legend('Bounded','Location','Northwest')

if pres_save
    %print(gcf,'-dpdf','var2bound-pres.pdf', '-image', '-r600')
    saveas(gcf,'var2bound-pres.pdf')
end

% Plot bounding region (across b011,b022 pairs)
plotdim = [b011_pts,b022_pts];
figpos = [1,5,6.5,7];
pad.topmarg = .03;
pad.leftmarg = .06;
pad.vspace = .07;
pad.hspace = .06;
pad.rightmarg = .02;
pad.botmarg = .05;
figure
set(gcf,'Position',figpos,...
    'PaperPosition',[0 0 figpos(3:4)],...
    'PaperSize',figpos(3:4));
for isubplot = 1:prod(plotdim)
    [icol,irow] = ind2sub(plotdim([2 1]),isubplot);
    spec = ['$(b_0^{11},b_0^{22}) = (' num2str(b011_grid(irow)) ',' num2str(b022_grid(icol)) ')$'];
    subplot(plotdim(1),plotdim(2),isubplot); hold on; box on;
    cond_tmp = cond(irow,icol,:,:);
    b012_gr_tmp = b012_gr(irow,icol,:,:);
    b021_gr_tmp = b021_gr(irow,icol,:,:);
    xaxis = b012_gr_tmp(cond_tmp==1);
    yaxis = b021_gr_tmp(cond_tmp==1);
    scatter(xaxis(:),yaxis(:),2,'.',...
        'MarkerFaceColor',gray,'MarkerEdgeColor',gray);
    title(spec,'Interpreter','latex'); 
    if icol==1
        ylabel('$b_0^{21}$','interpreter','latex')
    end
    if irow==b011_pts
        xlabel('$b_0^{12}$','interpreter','latex')
    end
    xlim([-limit,limit]); ylim([-limit,limit])
    xticks(-limit:1:limit); yticks(-limit:1:limit)
    yline(0,'LineWidth',1);
    xline(0,'LineWidth',1);
    set(gca, 'layer', 'top');
end
subplotpad(gcf,pad); 

% Save
if save_fig==1  
    print(gcf,'-dpdf','var2bound.pdf', '-image', '-r600')
end