function pf = epf(O,P,S,G,pf)

% Shock realizations
lnepArr3 = G.lne_nodes(:,ones(O.epssiga_pts,1),ones(O.epsa_pts,1));
lnsigapArr3 = permute(repmat(G.epssiga_nodes,[1,O.epse_pts,O.epsa_pts]),[2,1,3]);
epsaArr3 = permute(repmat(G.epsa_nodes,[1,O.epse_pts,O.epssiga_pts]),[2,3,1]);

% Disaster risk process
epArr3 = exp(lnepArr3);
ThetapArr3 = 1*(epArr3>=P.estar)+P.theta*(epArr3<P.estar);

% States and policy functions
lna = G.lna_gr;
k = G.k_gr;
n = pf.n;
a = exp(lna);

% Production function
y = a.*k.^P.alpha.*n.^(1-P.alpha);
% Firm FOC laboe
w = (1-P.alpha)*y./n;
% HH FOC labor
c = S.eta*w.*(1-n)/(1-S.eta);
% ARC
i = y-c;
i = max(0,i);

% Moments
pf.uy = zeros(G.griddim);
for inode = 1:G.nodes
    % Integration weights
    lne_weightVec = G.lne_weight(G.lne_gr(inode) == G.lne_grid,:)';
    epssiga_weightVec = G.epssiga_weight(G.lnsiga_gr(inode) == G.lnsiga_grid,:)';
    epsa_weightVec = G.epsa_weight(G.lna_gr(inode) == G.lna_grid,:)';        
    lne_weightArr3 = lne_weightVec(:,ones(O.epssiga_pts,1),ones(O.epsa_pts,1));        
    lnsiga_weightArr3 = permute(epssiga_weightVec(:,ones(O.epse_pts,1),ones(O.epsa_pts,1)),[2,1,3]);
    lna_weightArr3 = permute(epsa_weightVec(:,ones(O.epse_pts,1),ones(O.epssiga_pts,1)),[2,3,1]);
    weightArr3 = lne_weightArr3.*lnsiga_weightArr3.*lna_weightArr3;

    % Technology process
    lnapArr3 = (1-P.rhoa)*log(P.a) + P.rhoa*G.lna_gr(inode) ...
        + exp(G.lnsiga_gr(inode))*epsaArr3;      
    apArr3 = exp(lnapArr3);
    
    % Law of motion for capital
    kpArr3 = ThetapArr3*((1-P.delta)*k(inode) + i(inode));
    
    % Interpolate
    npArr3 = zeros(O.epse_pts,O.epssiga_pts,O.epsa_pts);
    npArr3(:) = Fallterp41(...
        G.lne_grid,G.lnsiga_grid,G.lna_grid,G.k_grid,...
        lnepArr3(:),lnsigapArr3(:),lnapArr3(:),kpArr3(:),...
        pf.n);

    % Production function
    ypArr3 = apArr3.*kpArr3.^P.alpha.*npArr3.^(1-P.alpha);
    
    % Output uncertainty
    Elogy = sum(weightArr3.*log(ypArr3),'all');
    pf.uy(inode) = sqrt(sum(weightArr3.*(log(ypArr3)-Elogy).^2,'all'));
end