function R = eqm(pf0,state,O,P,S,G,pf,...
    lnepArr3,lnsigapArr3,lnapArr3,ThetapArr3,weightArr3)

% Initialize output
R = zeros(size(pf0));
ncol = size(R,2);

% States
lna = state(1);             % Technology current period
k = state(2);           	% Capital current period
a = exp(lna);
apArr3 = exp(lnapArr3);

for icol = 1:ncol
    % Policy Function(s)
    n = pf0(1,icol);        % Hours current period
    J = pf0(2,icol);        % Value function current period
    %----------------------------------------------------------------------
    % Solve for variables
    %----------------------------------------------------------------------
    % Production function
    y = a*k^P.alpha*n^(1-P.alpha);
    % Firm FOC laboe
    w = (1-P.alpha)*y/n;
    % HH FOC labor
    c = S.eta*w*(1-n)/(1-S.eta);
    % ARC
    i = y-c;
    i = max(0,i);
    % Law of motion for capital
    kpArr3 = ThetapArr3*((1-P.delta)*k + i);
    % Utility
    u = c^S.eta*(1-n)^(1-S.eta);
    %----------------------------------------------------------------------
    % Interpolate
    %----------------------------------------------------------------------    
    npArr3 = zeros(O.epse_pts,O.epssiga_pts,O.epsa_pts);
    JpArr3 = zeros(O.epse_pts,O.epssiga_pts,O.epsa_pts);
    [npArr3(:),JpArr3(:)] = Fallterp42(...
        G.lne_grid,G.lnsiga_grid,G.lna_grid,G.k_grid,...
        lnepArr3(:),lnsigapArr3(:),lnapArr3(:),kpArr3(:),...
        pf.n,pf.J);
    %----------------------------------------------------------------------
    % Solve for variables inside expectations
    %----------------------------------------------------------------------
    % Production function
    ypArr3 = apArr3.*kpArr3.^P.alpha.*npArr3.^(1-P.alpha);
    % Firm FOC laboe
    wpArr3 = (1-P.alpha)*ypArr3./npArr3;
    % HH FOC labor
    cpArr3 = S.eta*wpArr3.*(1-npArr3)/(1-S.eta);
    % Rental rate
    rkpArr3 = P.alpha*ypArr3./kpArr3;
    % Utility
    upArr3 = cpArr3.^S.eta.*(1-npArr3).^(1-S.eta);    
    % Risk aggregator
    if P.gamma ~= 1
        z = sum(weightArr3.*JpArr3.^(1-P.gamma),'all')^(1/(1-P.gamma));
    elseif P.gamma == 1
        z = exp(sum(weightArr3.*log(JpArr3),'all'));
    end
    % SDF
    xpArr3 = P.beta*(c./cpArr3).*(upArr3./u).^(1-P.psi).*(JpArr3/z).^(1/P.psi-P.gamma); 
    %----------------------------------------------------------------------
    % Expectations
    %----------------------------------------------------------------------
    Einv = sum(weightArr3.*xpArr3.*ThetapArr3.*(rkpArr3+1-P.delta),'all');
    %----------------------------------------------------------------------
    % Euler equations
    %----------------------------------------------------------------------
    % Firm FOC capital
    R(1,icol) = 1-Einv;
    % Value function
    if P.psi ~= 1
        R(2,icol) = J^(1-1/P.psi) - (1-P.beta)*u^(1-1/P.psi) - P.beta*z^(1-1/P.psi);
    elseif P.psi == 1
        R(2,icol) = J-u^(1-P.beta)*z^P.beta;
    end   
end