function G = grids(O,P)

% G = grids(O,P)
%   Constructs an evenly-spaced discretized state space
% Inputs:
%	O :   Structure of user-specified options
%	P :   Structure of parameters
% Output:
%   G :   Structure of grid points

%--------------------------------------------------------------------------
% Nodes and weights for numerical integration
%--------------------------------------------------------------------------
% Disaster risk process
[G.lne_nodes,G.lne_weight] = rouwenhorst(log(P.e),P.rhoe,P.sige,O.lne_pts);
% Rouwenhorst for technology shock volatility
[G.epssiga_nodes,G.epssiga_weight] = rouwenhorst(log(P.siga),P.rhosv,P.sigsv,O.epssiga_pts);
% Rouwenhorst for technology shock
[G.epsa_nodes,G.epsa_weight] = rouwenhorst(0,0,1,O.epsa_pts);
%--------------------------------------------------------------------------
% Set up grids by discretizing the state space
%--------------------------------------------------------------------------
% Define grid points for disaster risk shock
G.lne_grid = G.lne_nodes';
% Define grid points for technology shock volatility
G.lnsiga_grid = G.epssiga_nodes';
% Define grid points for technology level
G.lna_grid = linspace(O.lnamin,O.lnamax,O.lna_pts);
% Define grid points for capital
G.k_grid = linspace(O.kmin,O.kmax,O.k_pts);
%--------------------------------------------------------------------------
% Construct grid arrays
%--------------------------------------------------------------------------
[G.lne_gr,G.lnsiga_gr,G.lna_gr,G.k_gr] = ndgrid(G.lne_grid,G.lnsiga_grid,G.lna_grid,G.k_grid);
G.nodes = numel(G.lnsiga_gr);
G.griddim = size(G.lnsiga_gr);