function [pf,eu] = guess(P,S,G,V)

% pf = guess(P,S,G)
%   Sets the initial policy functions
% Inputs:
%   P : structure of parameters
%   S : structure of steady state values
%   G : structure of grids
% Outputs:
%  pf : structure of policy functions

%----------------------------------------------------------------------
% log-linear solution
%----------------------------------------------------------------------
[T,~,eu] = linmodel(P,S,V);
disp(['eu=[' num2str(eu') ']'])

% Transform discretized state space
e_gr_per = G.lne_gr - log(P.e);
a_gr_per = G.lna_gr - log(P.a);
k_gr_per = log(G.k_gr) - log(S.k);

% Calculate linear policy functions
linpf_n = zeros(G.griddim);
linpf_J = zeros(G.griddim);
states = [e_gr_per(:),a_gr_per(:),k_gr_per(:)]';
linpf_n(:) = T(V.n,[V.e,V.a,V.k])*states;
linpf_J(:) = T(V.J,[V.e,V.a,V.k])*states;

% Convert back levels
pf.n = P.n*(1+linpf_n);
pf.J = S.J*(1+linpf_J);