% Generalized IRFs
clear; clc; close all;
plotops; format long
saving = 1;
loadmat = 0;

% Solution options
mod = 'qr';
fname = ['mats/Msolution-dens-' mod '.mat'];
disp(fname)

% GIRF options
nplot = 41+1;
nsims = 1e6;
nblocks = 250;
% 	Plot variables
V = variables;
varidx = [V.y,V.uy];
varstrs = {'Output','Uncertainty'};
plottypes = [1,1];
%   Shocks
shockstrs = {'DR Shock','Vol Shock','TFP Shock'};
nshocks = numel(shockstrs);
shockSDs = [1,1,1];

% Set random number seed and draw shocks
nsimsperblock = nsims/nblocks;
mtstream = RandStream('mt19937ar','seed',12345);
RandStream.setGlobalStream(mtstream);
nburn = 1e4;
shockslong = randn(nburn+1e4,1,nshocks);
shockshort = randn(nplot,nsimsperblock,nshocks,nblocks);

if loadmat==0
    % Monte Carlo Simulations
    girf = NaN(nplot,V.nplotvar,nshocks,2);
    
    % Load solution
    load(fname);

    % Ergodic distribution draws
    disp('Simulating ergodic distribution')
    sims = simulation(pf,P,S,G,V,shockslong);  
    sims = sims(nburn+1:end,:);
    ergmean = mean(sims,1)';
    simyg = sims(:,V.yg);
    SDyg = std(simyg);

    for ishock = 1:nshocks
        disp(shockstrs{ishock})
        impulse = shockshort;
        impulse(2,:,ishock,:) = shockSDs(ishock);
        girf1 = NaN(nplot,V.nplotvar,nblocks);
        girf2 = NaN(nplot,V.nplotvar,nblocks);
        parfor iblock = 1:nblocks
            % 	Baseline
            girf1(:,:,iblock) = mean(simulation(...
                pf,P,S,G,V,shockshort(:,:,:,iblock),...
                ergmean),3,'omitnan');
            %	Impulse
            girf2(:,:,iblock) = mean(simulation(...
                pf,P,S,G,V,impulse(:,:,:,iblock),...
                ergmean),3,'omitnan');
        end
        girf(:,:,ishock,1) = mean(girf1,3,'omitnan');
        girf(:,:,ishock,2) = mean(girf2,3,'omitnan');
    end
    if saving==1
        save(['mats/girf-' mod  '.mat'],...
            'girf','V','nsims','ergmean','SDyg')
    end
else
    load(['mats/girf-' mod '.mat'])
end

%% Figures
ifig = 0;
% Plot options
xaxis = 0:nplot-2;
plotdim = [numel(varidx),nshocks];
ishockorder = [1,2,3];
%   Paper version
ifig=ifig+1;
figpos{ifig} = [1,1,6.5,3.75];
pad(ifig).topmarg = .06;
pad(ifig).leftmarg = .08;
pad(ifig).vspace = .1;
pad(ifig).hspace = .07;
pad(ifig).rightmarg = .02;
pad(ifig).botmarg = .11;
legadj(ifig) = .1;
fontsize(ifig) = 10;
figtype{ifig} = '';
%   Presentation version
ifig=ifig+1;
figpos{ifig} = [8,1,4.5,3];
pad(ifig).topmarg = .12;
pad(ifig).leftmarg = .085;
pad(ifig).vspace = .08;
pad(ifig).hspace = .07;
pad(ifig).rightmarg = .02;
pad(ifig).botmarg = .09;
legadj(ifig) = .08;
fontsize(ifig) = 7;
figtype{ifig} = '_pres';

% Plot selected variables
for ifig = 1:1%numel(figtype)
    figure
    set(gcf,'Position',figpos{ifig},...
        'PaperPosition',[0 0 figpos{ifig}(3:4)],...
        'PaperSize',figpos{ifig}(3:4));
    for isubplot = 1:prod(plotdim)
        [icol,irow] = ind2sub(plotdim([2 1]),isubplot);
        ivar = irow; ishock = ishockorder(icol);
        subplot(plotdim(1),plotdim(2),isubplot); grid on; hold on; box on;
        plotdata = squeeze(girf(2:nplot,varidx(ivar),ishock,:));
        if plottypes(ivar) == 1
            yaxis = 100*(log(plotdata(:,2)) - log(plotdata(:,1)));            
        elseif plottypes(ivar) == 2
            yaxis = 100*(plotdata(:,2) - plotdata(:,1));
        elseif plottypes(ivar) == 3
            yaxis = 100*(plotdata(:,2) - plotdata(:,1))/SDyg;
        end
        plot(xaxis,yaxis,'k-'); 
        set(gca,'xlim',xaxis([1 end]),'xtick',xaxis(1):10:xaxis(end));
        if icol == 1
            ylabel(varstrs{ivar});
        end
        if irow == 1
            title(shockstrs{ishock})
        end
        if irow == plotdim(1)
            xlabel('Quarter');
        end
    end
    subplotpad(gcf,pad(ifig));
    set(findall(gcf,'-property','FontSize'),'FontSize',fontsize(ifig))
    set(findall(gcf,'-property','interpreter'),'interpreter','latex')
end