% Results
%close all; 
clear; clc;

% Solution options
lang = 'M';
runstr = 'dens'; 
mod = 'base';

% Simulation options
%   Long simulation
nburn = 1e3;
nperlong = 1e4;
nshocks = 3;
ps = [0.01,0.99];
%   Decomposition
nh = 1;
nsimsTVD = 5000;
%   Short simulations
npershort = nh+1;
F.nlagQ = 1;
F.ndataQ = 200;
F.nburn = 1e3;
F.nreps = 5001;

% Set random number seed and draw shocks
mtstream = RandStream('mt19937ar','seed',1234);
RandStream.setGlobalStream(mtstream);
shockslong = randn(nburn+nperlong,1,nshocks);
shocksshort = randn(F.nburn+F.ndataQ,F.nreps,nshocks);
shocksTVD = randn(npershort,nsimsTVD);

% Load solution
fname = ['mats/' lang 'solution-' runstr '-' mod '.mat'];
disp(fname); load(fname)

% Simulation results
disp('    Long simulations')
sims = simulation(pf,P,S,G,V,shockslong);
sims = sims(nburn+1:end,:);
ergmean = mean(sims,1)';
disater_per = 100*nnz(sims(:,V.Theta)<1)/nperlong;
disp(['    disater_per = ' num2str(disater_per)])

%       Disaster risk shock
boundsper = G.lne_grid([1,end]);
medpsper = quantile(sims(:,V.e),ps);
perout = sims(:,V.e) <= min(G.lne_grid) | sims(:,V.e) >= max(G.lne_grid);
if medpsper(1) < boundsper(1) || medpsper(2) > boundsper(2)
    warning('       e is out of bounds')
end
disp(['     e bounds:  ',num2str(boundsper)])
disp(['     e ergodic: ',num2str(medpsper)])
disp(['     e % out:   ',num2str(100*mean(perout))])
%       TFP Volatility shock
boundsper = G.lnsiga_grid([1,end]);
medpsper = quantile(sims(:,V.siga),ps);
perout = sims(:,V.siga) <= min(G.lnsiga_grid) | sims(:,V.siga) >= max(G.lnsiga_grid);
if medpsper(1) < boundsper(1) || medpsper(2) > boundsper(2)
    warning('       siga is out of bounds')
end
disp(['     siga bounds:  ',num2str(boundsper)])
disp(['     siga ergodic: ',num2str(medpsper)])
disp(['     siga % out:   ',num2str(100*mean(perout))])
%       TFP shock
boundsper = G.lna_grid([1,end]);
medpsper = quantile(sims(:,V.a),ps);
perout = sims(:,V.a) <= min(G.lna_grid) | sims(:,V.a) >= max(G.lna_grid);
if medpsper(1) < boundsper(1) || medpsper(2) > boundsper(2)
    warning('       a is out of bounds')
end
disp(['     a bounds:  ',num2str(boundsper)])
disp(['     a ergodic: ',num2str(medpsper)])
disp(['     a % out:   ',num2str(100*mean(perout))])
%	Capital
boundsper = G.k_grid([1,end])./S.k-1;
medpsper = quantile(sims(:,V.k),ps)./S.k-1;
perout = sims(:,V.k) <= min(G.k_grid) | sims(:,V.k) >= max(G.k_grid);
if medpsper(1) < boundsper(1) || medpsper(2) > boundsper(2)
	warning('       k is out of bounds')
end
disp(['     k bounds:  ',num2str(boundsper)])
disp(['     k ergodic: ',num2str(medpsper)])
disp(['     k % out:   ',num2str(100*mean(perout))])
disp('')

%% Compute short-sample moments
disp('	Short-sample simulations...');
sims = simulation(pf,P,S,G,V,shocksshort);
sims = sims(F.nburn+1:end,:,:);
M = momcalc(V,F,sims);

% Load data moments
load('../../Data/moments.mat')

% Display key moments 
mom_varidx = [V.y,V.uy];
mom_cstrings = {'Data','Model'};
mom_rstrings = {'SD(y)';'SD(a)';'AC(y)';'AC(a)'};
dsge_moms = [M.sd_yf,M.sd_af,M.ac_yf,M.ac_af]';
tabmoms = array2table([data_moms,dsge_moms]);
tabmoms = varfun(@(x) num2str(x, ['%',sprintf('.%df',1)]), tabmoms);
tabmoms.Properties.VariableNames = mom_cstrings;
tabmoms.Properties.RowNames = mom_rstrings;
disp(tabmoms)

%% Total variance decomposition
disp('	TVD'); 
Eysim_exelev = NaN(nh,V.nplotvar,nsimsTVD);
Vysim_exelev = NaN(nh,V.nplotvar,nsimsTVD);
Eysim_exalev = NaN(nh,V.nplotvar,nsimsTVD);
Vysim_exalev = NaN(nh,V.nplotvar,nsimsTVD);
Eysim_exavol = NaN(nh,V.nplotvar,nsimsTVD);
Vysim_exavol = NaN(nh,V.nplotvar,nsimsTVD);
for isim = 1:nsimsTVD
    % Simulate
    %   Fixed shocks to disaster risk
    shocks = cat(3,repmat(shocksTVD(:,isim),[1,nsimsTVD]),shocksTVD,shocksTVD);
    ysim_Felev = simulation(pf,P,S,G,V,shocks,ergmean);
    %   Fixed shocks to TFP volatility
    shocks = cat(3,shocksTVD,repmat(shocksTVD(:,isim),[1,nsimsTVD]),shocksTVD);
    ysim_Favol = simulation(pf,P,S,G,V,shocks,ergmean);
    %   Fixed shocks to TFP level
    shocks = cat(3,shocksTVD,shocksTVD,repmat(shocksTVD(:,isim),[1,nsimsTVD]));
    ysim_Falev = simulation(pf,P,S,G,V,shocks,ergmean);
    % Integrate
    %   Across all shocks, except disaster risk
    Eysim_exelev(:,:,isim) = mean(ysim_Felev(2:end,:,:),3,'omitnan');
    Vysim_exelev(:,:,isim) = var(ysim_Felev(2:end,:,:),[],3,'omitnan');
    %   Across all shocks, except TFP vol
    Eysim_exavol(:,:,isim) = mean(ysim_Favol(2:end,:,:),3,'omitnan');
    Vysim_exavol(:,:,isim) = var(ysim_Favol(2:end,:,:),[],3,'omitnan');
    %   Across all shocks, except TFP level
    Eysim_exalev(:,:,isim) = mean(ysim_Falev(2:end,:,:),3,'omitnan');
    Vysim_exalev(:,:,isim) = var(ysim_Falev(2:end,:,:),[],3,'omitnan');
end
% Integrate
%   Across TFP level shocks
TE_exalev = mean(Vysim_exalev,3)';
DE_alev = var(Eysim_exalev,[],3)';
%   Across TFP volatility shock
TE_exavol = mean(Vysim_exavol,3)';
DE_avol = var(Eysim_exavol,[],3)';
%   Across disaster risk shocks
TE_exelev = mean(Vysim_exelev,3)';
DE_elev = var(Eysim_exelev,[],3)';
% Direct Effect Variance Contribution
avol_Direct = 100*DE_avol./(TE_exavol+DE_avol);
alev_Direct = 100*DE_alev./(TE_exalev+DE_alev);
elev_Direct = 100*DE_elev./(TE_exelev+DE_elev);

% Display TVD results 
tvd_varidx = [V.y,V.uy];
tvd_cstrings = {'Output','Output Uncertainty'};
tvd_rstrings = {'TFP Volatility Shock','TFP Level Shock','DR Level Shock'};
tab_out = [avol_Direct(tvd_varidx,nh)';
           alev_Direct(tvd_varidx,nh)';
           elev_Direct(tvd_varidx,nh)'];
tabvardecomp = array2table(tab_out);
tabvardecomp = varfun(@(x) num2str(x, ['%',sprintf('.%df',1)]), tabvardecomp);
tabvardecomp.Properties.VariableNames = tvd_cstrings;
tabvardecomp.Properties.RowNames = tvd_rstrings;
disp(tabvardecomp)

% Display TVD results three variables 
tvd_varidx = [V.y,V.uy,V.i];
tvd_cstrings = {'Output','Output Uncertainty','Investment'};
tvd_rstrings = {'TFP Volatility Shock','TFP Level Shock','DR Level Shock'};
tab_out = [avol_Direct(tvd_varidx,nh)';
           alev_Direct(tvd_varidx,nh)';
           elev_Direct(tvd_varidx,nh)'];
tabvardecomp = array2table(tab_out);
tabvardecomp = varfun(@(x) num2str(x, ['%',sprintf('.%df',1)]), tabvardecomp);
tabvardecomp.Properties.VariableNames = tvd_cstrings;
tabvardecomp.Properties.RowNames = tvd_rstrings;
disp(tabvardecomp)