clear
clc
saving = 1;

% Solution options
O.mod = 'base';
O.tol = 1e-4;
%   Grids
%       States
O.lne_pts = 15;           % Disaster risk state
O.lnsiga_pts = 7;        % TFP Volatility Shock
O.lna_pts = 7;      	 % TFP Level Shock
O.k_pts = 7;            % Capital
%       Shocks
O.epse_pts = O.lne_pts; 
O.epsa_pts = O.lna_pts;
O.epssiga_pts = O.lnsiga_pts;
%       Bounds
O.aper = 0.075;
O.kperl = 0.6;
O.kperh = 0.1;

% Load structures
V = variables;
P = parameters(O);
S = steadystate(V,P);

% Load grids
O.lnamin = log((1-O.aper)*P.a);
O.lnamax = log((1+O.aper)*P.a);
O.kmin = (1-O.kperl)*S.k;
O.kmax = (1+O.kperh)*S.k;
G = grids(O,P);

% Solve model
%   Linear guess
pf = guess(P,S,G,V);
linpf = pf;
%   Nonlinear solution
[pf,converged,reason] = pf_alg(O,P,S,G,pf);
disp(['converged = ',num2str(converged)])
%   Uncertainty policy functions
pf = epf(O,P,S,G,pf);

% Save results
if saving && converged
    fname = ['mats/Msolution-test-' O.mod '.mat'];
    save(fname,'O','P','S','G','V','pf','linpf');
end