% Compute RMSEs and plot IRFs
if exist('genresults','var')
    saving = 0;
    plotting = 0;       
else
    clear; clc; close all; 
    mod = 'base';
    saving = 0;
    plotting = 1;    
    plotverson = '';    
end
% Cases flags
hvarflags = 0;
cholflags = 1; 
carrieroflags = NaN; %[cholesky;max share]
ntargets = NaN;      %Number of targets

%% Load mats
%   VAR responses
varslist = {'siga','uy','y'};

norders = numel(carrieroflags);
for iorder = 1:norders
    if cholflags(iorder)==1
        load(['mats/var-' varslist{1} '-' varslist{2} '-' varslist{3} '-' mod '-chol-news.mat']);
    else
        load(['mats/var-' varslist{1} '-' varslist{2} '-' varslist{3} '-' mod ...
            '-ntarget' num2str(ntargets(iorder)) '-hvar' num2str(hvarflags(iorder)) ...
            '-carr' num2str(carrieroflags(iorder)) '-news.mat']);
    end
    irfs_all{iorder} = squeeze(irfs);
    varidx_all{iorder} = varidx;
end
%   Model responses
load(['mats/girf-' mod '.mat'])

%% Root mean squared error
hs = 1:41; ilag = (lags == 4);
dsgeuyimpact = 100*(log(girf(2,V.uy,iavolidx,2))-log(girf(2,V.uy,iavolidx,1)));
dsgeirf_tmp = squeeze(girf(hs+1,V.y,iavolidx,:));
dsgeirf = 100*diff(log(dsgeirf_tmp),[],2);
rmses = NaN(1,norders);
for iorder = 1:norders
    iy = find(strcmp(varidx_all{iorder},'y'));
    iuy = find(strcmp(varidx_all{iorder},'siga'));
    varirf = irfs_all{iorder}(hs,iy,iuy,ilag);
    iuncertainty = find(strcmp(varidx_all{iorder},'uy'));
    varuyimpact = irfs_all{iorder}(1,iuncertainty,iuy,ilag);
    rescale(iorder) = dsgeuyimpact/varuyimpact;
    bias = rescale(iorder)*varirf-dsgeirf;
    rmses(iorder) = sum(abs(bias));
end

%% Plot impulse responses
if plotting == 1
    plotops; nplot = 41;
    figure
    fontsize = 8;
    ilag = (lags == 4);
    xaxis = (1:nplot)-1;
    legstrs = {'Cholesky','Population'};
    % figpos = [1,5,2.1,2.1];
    % axispos = [.12,.1,.82,.85];
    figpos = [1,5,3.2,3.2];
    axispos = [.09,.07,.86,.90];
    if strcmp(plotverson,'P')
        axispos = [.12,.1,.82,.82];
    end
    plottype = {'r--','b-.','k-'};
    set(gcf,'Position',figpos,...
        'PaperPosition',[0,0,figpos(3:4)],...
        'PaperSize',figpos(3:4),...
        'name',mod);
    set(gca,'Position',axispos);
    hold on; box on; grid on;
    % VAR responses
    for iorder = 1:norders
        iy = find(strcmp(varidx_all{iorder},'y'));
        iuy = find(strcmp(varidx_all{iorder},'siga'));
        yaxis = squeeze(rescale(iorder)*irfs_all{iorder}(1:nplot,iy,iuy,ilag)); 
        plot(xaxis,yaxis,plottype{iorder}); 
    end
    % Model responses
    plotdata = squeeze(girf(2:nplot+1,V.y,iavolidx,:));
    yaxis = 100*diff(log(plotdata),[],2);
    plot(xaxis,yaxis,plottype{norders+1});
    ylim([-0.05,0.05]); yticks(-0.05:0.01:0.05)
    % Legend
    legend(legstrs,'Location','SouthEast','FontSize',fontsize)
    if strcmp(plotverson,'P')
        title('Output ($\%$)','interpreter','latex','FontSize',fontsize)
    end
    set(findall(gcf,'-property','FontSize'),'FontSize',fontsize)
    set(findall(gcf,'-property','interpreter'),'interpreter','latex')
    % Save
    if saving==1
        savename = ['responses-' mod];
        if strcmp(plotverson,'P')
            savename = [savename '-' plotverson];
        end
        saveas(gcf,['results/' savename '.pdf'])
    end
end