function sims = simulation(pf,P,S,G,V,shocks,varargin)

%----------------------------------------------------------------------
% Initialize Simulation
%----------------------------------------------------------------------
npers = size(shocks,1);
nsims = size(shocks,2);
stateidx = [V.e,V.siga,V.a,V.k,V.i,V.y];
detss = [log(P.e),log(P.siga),log(P.a),S.k,S.i,S.y]';
sims = zeros(npers,V.nplotvar,nsims);
% Initial conditions
if isempty(varargin)
    sims(1,stateidx,:) = repmat(detss,[1,nsims]);
else
    if size(varargin{1},2) == 1
        sims(1,:,:) = repmat(varargin{1},[1,nsims]);
    else
        sims(1,:,:) = varargin{1};
    end
end
% Shocks
epseTemp = permute(shocks(:,:,1),[1 3 2]);
epssigaTemp = permute(shocks(:,:,2),[1 3 2]);
epsaTemp = permute(shocks(:,:,3),[1 3 2]);

%----------------------------------------------------------------------
% Simulate time series
%----------------------------------------------------------------------
simsisbad = false(nsims,1);
for t = 2:npers
    % Disaster risk
    sims(t,V.e,:) = (1-P.rhoe)*log(P.e)+P.rhoe*sims(t-1,V.e,:) + P.sige*epseTemp(t,1,:);
    e = exp(sims(t,V.e,:));
    sims(t,V.Theta,:) = 1*(e>=P.estar) + P.theta*(e<P.estar);
    % Technology
    sims(t,V.siga,:) = (1-P.rhosv)*log(P.siga) + P.rhosv*sims(t-1,V.siga,:) + P.sigsv*epssigaTemp(t,1,:);
    sims(t,V.a,:) = (1-P.rhoa)*log(P.a) + P.rhoa*sims(t-1,V.a,:) + exp(sims(t-1,V.siga,:)).*epsaTemp(t,1,:);
    a = exp(sims(t,V.a,:));
    % IV
    sims(t,V.ivepsav,:) = P.sigsv*epssigaTemp(t,1,:);
    % Law of motion for capital
    sims(t,V.k,:) = sims(t,V.Theta,:).*((1-P.delta).*sims(t-1,V.k,:) + sims(t-1,V.i,:));         
    % Evaluate policy functions
    [sims(t,V.n,:),sims(t,V.uy,:)] = ...
        Fallterp42( ...
            G.lne_grid,G.lnsiga_grid,G.lna_grid,G.k_grid,...
            squeeze(sims(t,V.e,:)),squeeze(sims(t,V.siga,:)),...
            squeeze(sims(t,V.a,:)),squeeze(sims(t,V.k,:)),...
            pf.n,pf.uy);
    % Production function
    sims(t,V.y,:) = a.*sims(t,V.k,:).^P.alpha.*sims(t,V.n,:).^(1-P.alpha);
    % Firm FOC laboe
    sims(t,V.w,:) = (1-P.alpha)*sims(t,V.y,:)./sims(t,V.n,:);
    % HH FOC labor
    sims(t,V.c,:) = S.eta*sims(t,V.w,:).*(1-sims(t,V.n,:))/(1-S.eta);
    % ARC
    i = sims(t,V.y,:)-sims(t,V.c,:); 
    sims(t,V.i,:) = max(0,i);    
    % Output growth
    sims(t,V.yg,:) = log(sims(t,V.y,:))-log(sims(t-1,V.y,:));
    
    % Check for complex sims
    isbad = squeeze(any(imag(sims(t,:,:)) ~= 0,2));
    if any(isbad)
        % Track complex sims
        simsisbad = simsisbad | isbad;
        % Reset complex sims to steady state
        sims(t,:,isbad) = 0;
        sims(t,stateidx,isbad) = repmat(detss,[1,sum(isbad)]);
    end
end
if any(simsisbad)
    disp(['% of sims complex = ',num2str(100*sum(simsisbad)/nsims)]);
    sims(:,:,simsisbad) = NaN;
end