function S = steadystate(V,P)

% S = steadystate(V,P)
%   Computes the deterministic steady state
% Input:
%   V : Structure of variables
%   P : Structure of parameters
% Output:
%   S : Structure of steady state values

% HH FOC capital
S.rk = 1/P.beta+P.delta-1;
% Production function and rental rate
S.y = (P.alpha/S.rk)^(P.alpha/(1-P.alpha))*P.n;
% Rental rate
S.k = P.alpha*S.y/S.rk;
% Firm FOC labor 
S.w = (1-P.alpha)*S.y/P.n;
% Capital LOM
S.i = P.delta*S.k;
% ARC
S.c = S.y-S.i;
% HH FOC labor
S.eta = S.c/(S.w*(1-P.n)+S.c);
% SDF
S.x = P.beta;
% Utility
S.u = S.c^S.eta*(1-P.n)^(1-S.eta);
% Value function
S.J = S.u;
% Risk aggregator
S.z = S.J;

% Vectorize
S.detss = NaN(V.nplotvar,1);
for ivar = 1:V.nplotvar
    plotname = V.plotnames{ivar};
    if isfield(P,plotname)
        S.detss(ivar) = P.(plotname);
    elseif isfield(S,plotname)
        S.detss(ivar) = S.(plotname);
    end
end