function [irfs,x,strucshocks,fevds] = var_maxshare(vardata,F,V,Ovar)

hvar = Ovar.hvar;
hirf = Ovar.hirf;
lag = Ovar.lag;
ntarget = Ovar.ntarget;
cholflag = Ovar.cholflag;
carrieroflag = Ovar.carrieroflag;
hvars_fevd = Ovar.hvars_fevd;
nhvars_fevd = Ovar.nhvars_fevd;
% ivaruy = Ovar.ivaruy;

% Seed
mtstream = RandStream('mt19937ar','seed',1);
RandStream.setGlobalStream(mtstream);

% Optimizer options
options = knitro_options('algorithm',5,'opttol',1e-8,...
        'outlev',0,'numthreads',1);
nguess = 10;

% Run loop over simulations  
K = size(vardata,2);
x = NaN(K*K,F.nreps);
fvals = NaN(1,F.nreps);
irfs = NaN(hirf+1,K,K,F.nreps); 
impact = ones(1,K);
for irep = 1:F.nreps
    disp(['irep = ' num2str(irep)])

    % Estimate unrestricted VAR
    [A,Sigma,U] = lsvarc(vardata,lag);
    P = chol(Sigma)';
    
    % Compute Cholesky Impulse Responses
    B = zeros(K,K*(hirf+1)); % Changed to hirf, with hirf > max(hvar(:))
    J = [eye(K) zeros(K,K*(lag-1))];
    for i = 0:hirf
        PHItemp = J*A^i*J';
        B(:,K*i+1:K*i+K) = PHItemp(1:K,1:K);
    end    
       
    % Solve for D matrix
    Dfull = zeros(K,K);
    if cholflag
        Dfull = eye(K);
    else
        % Solve for the unknown parameters       
        x0 = 0.5*rand(K*K,nguess);  
        fvalvec = NaN(1,nguess);
        xvec = NaN(K*K,nguess);
        parfor iguess=1:nguess
            [xtmp,fvaltmp,flag] = knitro_nlp(@(x)objfun(x,B,hvar,P,Sigma,K,ntarget),...
                x0(:,iguess),[],[],[],[],[],[],...
                @(x)nlcon(x,K,hvar,B,P,Sigma,ntarget,carrieroflag),[],options);
            if flag==0
                xvec(:,iguess) = xtmp;
                fvalvec(iguess) = fvaltmp;        
            end
        end        
        if all(isnan(fvalvec))
            continue
        else
            loc = find(fvalvec==min(fvalvec),1,'first');
            fvals(irep) = fvalvec(loc);
            x(:,irep) = xvec(:,loc)'; 
            disp(['fval = ' num2str(fvals(irep))])   
        end           
        for i = 1:K
            Dfull(:,i) = x(K*(i-1)+1:K*i,irep);
        end
    end
    
    % Structural shocks
    PDfull = P*Dfull;
    strucshocks = transpose(PDfull\U(1:K,:));
    
    % Structural IRFs
    % (vol, level, residual)
    for i = 0:hirf
        PHItemp = J*A^i*J';        
        %Dimensions: (horizion,variables,shocks,reps)
        irfs(i+1,:,:,irep) = PHItemp*P*Dfull.*repmat(impact,[K,1]);
    end
    % Check the uncertainty shock increases uncertainty on impact
    if irfs(1,1,1,irep)<0
        irfs(:,:,1,irep) = -irfs(:,:,1,irep);
    end    
    % Check the level shock increases output on impact
    if irfs(1,2,2,irep)<0
        irfs(:,:,2,irep) = -irfs(:,:,2,irep);
    end
    
    % Calculate FEVDs    
    fevds = fevd_func(F,B,P,Dfull,Sigma,nhvars_fevd,hvars_fevd,K);
end