function V = variables

% Variables: Names variables and assigns locations
% Output:
%     V     :   Structure of variable locations and names

% Variables names
V.names = { 'y'         %1  Output
            'k'         %2  Capital
            'c'         %3  Consumption
            'n'         %4  Hours
            'w'         %5  Wage rate
            'rk'        %6  Rental Rate
            'i'         %7  Investment
            'J'         %8  HH value function
            'x'         %9  Pricing Kernel                        
            'u'         %10 Utility function
            'z'         %11 Risk aggregator
            'e'         %12 Disaster risk
            'a'         %13 Technology
            };

% Variables titles
V.desc = {  'Output'
            'Capital'
            'Consumption'
            'Hours'
            'Wage Rate'
            'Rental Rate'
            'Investment'
            'Value Function'
            'Pricing Kernel'
            'Utility Function'
            'Risk Aggregator'
            'Disaster Risk'  
            'Technology'
         };

% Shocks
V.shocktypes = {'e','a'};
V.descshock = {'Disaster Risk','Technology'};
V.nshock = length(V.shocktypes);

% Forecast errors
V.foretypes = {'x','rk','J'};
V.nfore = length(V.foretypes);

% Variables names for nonlinear model
V.plotnames = [V.names
              'siga'
              'Theta'  
              'uy'
              'yg'
              'ivepsav'];
V.nplotvar = length(V.plotnames);

% Variables descriptions for nonlinear model
V.desc = [V.desc
          'SD Technology Shock'
          'Capital Quality'
          'Output Uncertainty'
          'Output Growth'
          'IV (shock)'];

% Add expectational dummies
for j = 1:V.nfore
    V.names = [V.names; ['e' V.foretypes{j}]];
end
V.nvar = length(V.names);
V.nstate = V.nvar-V.nfore;

% Establish variable index
%  Linear model
for j = 1:V.nvar
   V.(V.names{j}) = j;
end
%  Nonlinear model
for j = V.nvar-V.nfore+1:V.nplotvar
   V.(V.plotnames{j}) = j;
end
% Establish shock index
for j = 1:V.nshock
    V.("eps" + V.shocktypes{j}) = j;
end
% Establish forecast error index
for j = 1:V.nfore
    V.("fe" + V.foretypes{j}) = j;
end