function pf = epf(O,P,S,G,pf)

% Shock realizations
lnsigepArr3 = repmat(G.epssige_nodes,[1,O.epse_pts,O.epsa_pts]);
epseArr3 = permute(repmat(G.epse_nodes,[1,O.epssige_pts,O.epsa_pts]),[2,1,3]);
lnapArr3 = permute(repmat(G.lna_nodes,[1,O.epssige_pts,O.epse_pts]),[2,3,1]);

% States and policy functions
lna = G.lna_gr;
k = G.k_gr;
n = pf.n;
a = exp(lna);
apArr3 = exp(lnapArr3);

% Production function
y = a.*k.^P.alpha.*n.^(1-P.alpha);
% Firm FOC laboe
w = (1-P.alpha)*y./n;
% HH FOC labor
c = S.eta*w.*(1-n)/(1-S.eta);
% ARC
i = y-c;
i = max(0,i);

% Moments
pf.uy = zeros(G.griddim);
for inode = 1:G.nodes
    % Integration weights
    epssige_weightVec = G.epssige_weight(G.lnsige_gr(inode) == G.lnsige_grid,:)';
    epse_weightVec = G.epse_weight(G.lne_gr(inode) == G.lne_grid,:)';
    lna_weightVec = G.lna_weight(G.lna_gr(inode) == G.lna_grid,:)';      
    lnsige_weightArr3 = epssige_weightVec(:,ones(O.epse_pts,1),ones(O.epsa_pts,1));
    lne_weightArr3 = permute(epse_weightVec(:,ones(O.epssige_pts,1),ones(O.epsa_pts,1)),[2,1,3]);
    lna_weightArr3 = permute(lna_weightVec(:,ones(O.epssige_pts,1),ones(O.epse_pts,1)),[2,3,1]);
    weightArr3 = lnsige_weightArr3.*lne_weightArr3.*lna_weightArr3;

    % Disaster risk process
    lnepArr3 = (1-P.rhoe)*log(P.e) + P.rhoe*G.lne_gr(inode) ...
        + exp(G.lnsige_gr(inode))*epseArr3;
    epArr3 = exp(lnepArr3);
    ThetapArr3 = 1*(epArr3>=P.estar)+P.theta*(epArr3<P.estar); 
    
    % Law of motion for capital
    kpArr3 = ThetapArr3*((1-P.delta)*k(inode) + i(inode));
    
    % Interpolate
    npArr3 = zeros(O.epssige_pts,O.epse_pts,O.epsa_pts);
    npArr3(:) = Fallterp41(...
        G.lnsige_grid,G.lne_grid,G.lna_grid,G.k_grid,...
        lnsigepArr3(:),lnepArr3(:),lnapArr3(:),kpArr3(:),...
        pf.n);

    % Production function
    ypArr3 = apArr3.*kpArr3.^P.alpha.*npArr3.^(1-P.alpha);
    
    % Output uncertainty
    Elogy = sum(weightArr3.*log(ypArr3),'all');
    pf.uy(inode) = sqrt(sum(weightArr3.*(log(ypArr3)-Elogy).^2,'all'));
end