function G = grids(O,P)

% G = grids(O,P)
%   Constructs an evenly-spaced discretized state space
% Inputs:
%	O :   Structure of user-specified options
%	P :   Structure of parameters
% Output:
%   G :   Structure of grid points

%--------------------------------------------------------------------------
% Nodes and weights for numerical integration
%--------------------------------------------------------------------------
% Rouwenhorst for disaster risk shock volatility
[G.epssige_nodes,G.epssige_weight] = rouwenhorst(log(P.sige),P.rhosv,P.sigsv,O.epssige_pts);
% Rouwenhorst for disaster risk level
[G.epse_nodes,G.epse_weight] = rouwenhorst(0,0,1,O.epse_pts);
% Rouwenhorst for technology level
[G.lna_nodes,G.lna_weight] = rouwenhorst(log(P.a),P.rhoa,P.siga,O.lna_pts);
%--------------------------------------------------------------------------
% Set up grids by discretizing the state space
%--------------------------------------------------------------------------
% Define grid points for disaster risk shock volatility
G.lnsige_grid = G.epssige_nodes';
% Define grid points for disaster risk level
G.lne_grid = linspace(O.lnemin,O.lnemax,O.lne_pts);
% Define grid points for technology shock
G.lna_grid = G.lna_nodes';
% Define grid points for capital
G.k_grid = linspace(O.kmin,O.kmax,O.k_pts);
%--------------------------------------------------------------------------
% Construct grid arrays
%--------------------------------------------------------------------------
[G.lnsige_gr,G.lne_gr,G.lna_gr,G.k_gr] = ndgrid(G.lnsige_grid,G.lne_grid,G.lna_grid,G.k_grid);
G.nodes = numel(G.lnsige_gr);
G.griddim = size(G.lnsige_gr);