% Nonlinear constraints
function [c,ceq] = nlcon(x,K,hvar,B,P,Sigma,ntarget,carrieroflag)

qmat = zeros(K,K);
for i = 1:K
    qmat(:,i) = x((K*(i-1))+1:K*i);
end

% Calculate the variance shares
%   row: shock number; column: variable number
om = zeros(ntarget,K); 
if carrieroflag ==0
    for shocknum = 1:ntarget 
        for varnum = 1:1 
            top = 0;
            bottom = 0;
            for j=0:hvar(shocknum)
                Bvar = B(varnum,K*j+1:K*j+K); 
                top = top + (Bvar*P*qmat(:,shocknum))*(qmat(:,shocknum)'*P'*Bvar'); 
                bottom = bottom + Bvar*Sigma*Bvar';
                om(shocknum,varnum) = om(shocknum,varnum) + top/bottom;
            end
        end
    end
else
    for shocknum = 1:ntarget 
        for varnum = 1:K 
            top = 0;
            bottom = 0;
            for j=0:hvar(shocknum)
                Bvar = B(varnum,K*j+1:K*j+K); 
                top = top + (Bvar*P*qmat(:,shocknum))*(qmat(:,shocknum)'*P'*Bvar'); 
                bottom = bottom + Bvar*Sigma*Bvar';
                om(shocknum,varnum) = om(shocknum,varnum) + top/bottom;
            end
        end
    end
end

% Inequality constraints (x <= y)
if carrieroflag ==0
    c = [];
elseif carrieroflag==1
    c = NaN(1,ntarget*(K-1));
    if ntarget ==1
        c(1) = om(1,2) - om(1,1); 
        if K ==3
            c(2) = om(1,3) - om(1,1);
        end
    elseif ntarget ==2
        c(1) = om(1,2) - om(1,1); 
        c(2) = om(2,1) - om(2,2); 
        if K ==3
            c(3) = om(1,3) - om(1,1); 
            c(4) = om(2,3) - om(2,2); 
        end 
    elseif ntarget == 3
        c(1) = om(1,2) - om(1,1); 
        c(2) = om(1,3) - om(1,1);  
        c(3) = om(2,1) - om(2,2); 
        c(4) = om(2,3) - om(2,2); 
        c(5) = om(3,1) - om(3,3);
        c(6) = om(3,2) - om(3,3);
    else
        disp('Number of targets must be between 1 and 3')
        return
    end
end

% Equality constraints
if K ==2
    q1 = qmat(:,1);
    q2 = qmat(:,2);
    ceq(1) = sum(q1.^2)-1;
    ceq(2) = sum(q2.^2)-1;
    ceq(3) = q2'*q1;
    ceq(4) = q1(1)^2 + q2(1)^2 - 1;
    ceq(5) = q1(2)^2 + q2(2)^2 - 1;
    ceq(6) = q1(1)*q1(2) + q2(1)*q2(2);
elseif K ==3
    q1 = qmat(:,1);
    q2 = qmat(:,2);
    q3 = qmat(:,3);
    ceq(1) = sum(q1.^2)-1;
    ceq(2) = sum(q2.^2)-1;
    ceq(3) = sum(q3.^2)-1;
    ceq(4) = q2'*q1;
    ceq(5) = q3'*q1;
    ceq(6) = q2'*q3;
    ceq(7) = q1(1)^2 + q2(1)^2 + q3(1)^2 - 1;
    ceq(8) = q1(2)^2 + q2(2)^2 + q3(2)^2 - 1;
    ceq(9) = q1(3)^2 + q2(3)^2 + q3(3)^2 - 1;
    ceq(10) = q1(1)*q1(2) + q2(1)*q2(2) + q3(1)*q3(2);
    ceq(11) = q1(1)*q1(3) + q2(1)*q2(3) + q3(1)*q3(3);
    ceq(12) = q1(2)*q1(3) + q2(2)*q2(3) + q3(2)*q3(3);
end