function [pf,converged,reason] = pf_alg(O,P,S,G,pf)

% Preallocate arrays to store policy function updates
pf_n_up = zeros(G.griddim);
pf_J_up = zeros(G.griddim);

% Shock realizations
lnsigepArr3 = repmat(G.epssige_nodes,[1,O.epse_pts,O.epsa_pts]);
epseArr3 = permute(repmat(G.epse_nodes,[1,O.epssige_pts,O.epsa_pts]),[2,1,3]);
lnapArr3 = permute(repmat(G.lna_nodes,[1,O.epssige_pts,O.epse_pts]),[2,3,1]);

it = 1;                                 % Iteration Counter
converged = -1;                         % Convergence Flag
reason = 0; 							% Stopping reason
tstart = tic;                           % Job timer start
while converged == -1
    istart = tic;                       % Iteration timer start
    parfor inode = 1:G.nodes
        % Find optimal policy functions on each node
        start = [pf.n(inode),pf.J(inode)]';
        state = [G.lna_gr(inode),G.k_gr(inode)];
        
        % Integration weights
        epssige_weightVec = G.epssige_weight(G.lnsige_gr(inode) == G.lnsige_grid,:)';
        epse_weightVec = G.epse_weight(G.lne_gr(inode) == G.lne_grid,:)';
        lna_weightVec = G.lna_weight(G.lna_gr(inode) == G.lna_grid,:)';      
        lnsige_weightArr3 = epssige_weightVec(:,ones(O.epse_pts,1),ones(O.epsa_pts,1));
        lne_weightArr3 = permute(epse_weightVec(:,ones(O.epssige_pts,1),ones(O.epsa_pts,1)),[2,1,3]);
        lna_weightArr3 = permute(lna_weightVec(:,ones(O.epssige_pts,1),ones(O.epse_pts,1)),[2,3,1]);
        weightArr3 = lnsige_weightArr3.*lne_weightArr3.*lna_weightArr3;

        % Disaster risk process
        lnepArr3 = (1-P.rhoe)*log(P.e) + P.rhoe*G.lne_gr(inode) ...
            + exp(G.lnsige_gr(inode))*epseArr3;
        epArr3 = exp(lnepArr3);
        ThetapArr3 = 1*(epArr3>=P.estar)+P.theta*(epArr3<P.estar);
        
        % Approximate solution
        argzero = csolve('eqm',start,[],1e-4,10,state,...
            O,P,S,G,pf,lnsigepArr3,lnepArr3,lnapArr3,ThetapArr3,weightArr3);
        % Store updated policy functions
        pf_n_up(inode) = argzero(1);
        pf_J_up(inode) = argzero(2);
    end

    % Policy function distances
    dist_n = abs(pf_n_up./pf.n-1);
    dist_J = abs(pf_J_up./pf.J-1);

    % Maximum distance
    dist_max = max([dist_n(:)',dist_J(:)']);
    
    % Update policy functions
    pf.n = pf_n_up;
    pf.J = pf_J_up;

    % Stopping reasons
    if dist_max > 1
        reason = 1;
    elseif it > 5000
        reason = 2;
    end

    % Check convergence criterion
    if dist_max < O.tol
        converged = 1;
    elseif reason > 0
        converged = 0;
    end

    % Iteration Information
    it = itinfo(istart,tstart,1,it,dist_max);
end