clear
clc
saving = 1;

% Solution options
O.mod = 'base';
O.tol = 1e-4;
%   Grids
%       States
O.lnsige_pts = 7;        % Disaster risk volatility
O.lne_pts = 15;          % Disaster risk level
O.lna_pts = 7;      	 % TFP level
O.k_pts = 7;             % Capital
%       Shocks
O.epssige_pts = O.lnsige_pts;
O.epse_pts = O.lne_pts; 
O.epsa_pts = O.lna_pts;
%       Bounds
O.eper = 0.15;
O.kperl = 0.7;
O.kperh = 0.1;

% Load structures
V = variables;
P = parameters(O);
S = steadystate(V,P);

% Load grids
O.lnemin = log((1-O.eper)*P.e);
O.lnemax = log((1+O.eper)*P.e);
O.kmin = (1-O.kperl)*S.k;
O.kmax = (1+O.kperh)*S.k;
G = grids(O,P);

% Solve model
%   Linear guess
pf = guess(P,S,G,V);
linpf = pf;
%   Nonlinear solution
[pf,converged,reason] = pf_alg(O,P,S,G,pf);
disp(['converged = ',num2str(converged)])
%   Uncertainty policy functions
pf = epf(O,P,S,G,pf);

% Save results
if saving && converged
    fname = ['mats/Msolution-test-' O.mod '.mat'];
    save(fname,'O','P','S','G','V','pf','linpf');
end