% Simulate model and estimate VAR
clear; clc; close all;
warning('off','backtrace')

% Saving
saving = 1;

% VAR options
mod = 'qr';
firstvarlist = {'uy','y'};
middlevarlist = {'i','c','n','w'}; 
nfirstvar = size(firstvarlist,2);
nmiddlevar = size(middlevarlist,2);

lags = [4,8,16];   % Set number of lags
nlags = numel(lags);

% Simulation options
nreps = 1;
ndata= 1e6;
nburn = 1e4;

% FEVD horizons
hvars_fevd = [4,20,40,80];
nhvars_fevd = numel(hvars_fevd);
for ihvar=1:nhvars_fevd
    colstrs{ihvar} = ['h=' num2str(hvars_fevd(ihvar))];
end

% VAR indexes 
V = variables;

% Set random number seed and draw shocks
mtstream = RandStream('mt19937ar','seed',123456);
RandStream.setGlobalStream(mtstream);

% Draw shocks
shockslong = randn(nburn+ndata,V.nshock+1,nreps);

% Load solution
fname = ['mats/Msolution-dens-' mod '.mat'];
load(fname)

% Simulations
load(['mats/simdata-' O.mod '.mat'])
sims(sims(:,V.uy)<0,V.uy)=0.0001;
sims(isnan(sims(:,V.uy)),V.uy)=0.0001;
ievolidx = 1;
ielevidx = 2;
ialevidx = 3;

% Load population responses
load(['mats/girf-' mod '.mat'],'girf')
dsge_impact_uy = 100*(log(girf(2,V.uy,ievolidx,2))-log(girf(2,V.uy,ievolidx,1)));

% VAR
disp('Estimating VAR...');
hvar = 200;
K = 3;
irfs = NaN(hvar,K,3,nreps,nlags);
fevds = NaN(K,nhvars_fevd,nreps);
impact = ones(1,K);

for ivarfirst = 1:nfirstvar
    for ivarmiddle = 1:nmiddlevar
        firstvar = firstvarlist{ivarfirst};    
        middlevar = middlevarlist{ivarmiddle};   
        
        if strcmp(firstvar,'uy')
        lastvar = 'y';
        elseif strcmp(firstvar,'y')
        lastvar = 'uy';  
        end
        varidx = {firstvar,middlevar,lastvar};
        ivaruy = find(strcmp(varidx,'uy'));
        ivary = find(strcmp(varidx,'y'));

        for ilag = 1:nlags
            lag = lags(ilag);
            disp(['Order: [',varidx{1},', ',varidx{2},', ',varidx{3},'], lag' num2str(lag)])    
            corrarray = zeros(6,nreps);
            for irep = 1:nreps 
                % Output
                simy = 100*log(sims(2:end,V.y,irep));
                % Middle variable
                if strcmp(middlevar,'c')
                    simmid = 100*log(sims(2:end,V.c,irep));
                elseif strcmp(middlevar,'i')
                    simmid = 100*log(sims(2:end,V.i,irep));
                elseif strcmp(middlevar,'w')
                    simmid = 100*log(sims(2:end,V.w,irep));
                elseif strcmp(middlevar,'n')
                    simmid = 100*log(sims(2:end,V.n,irep));
                elseif strcmp(middlevar,'a')
                    simmid = 100*sims(2:end,V.a,irep);     
                end       
                % Uncertainty
                simuy = 100*log(sims(2:end,V.uy,irep));    

                % Assign data in VAR order
                if strcmp(firstvar,'uy')
                    vardata = [simuy,simmid,simy];
                elseif strcmp(firstvar,'y')
                    vardata = [simy,simmid,simuy];
                end

                % Estimate VAR
                [A,Sigma,U] = lsvarc(vardata,lag);
                P = chol(Sigma)';        

                % Compute Cholesky Impulse Responses
                B = zeros(K,K*(hvar+1));
                J = [eye(K) zeros(K,K*(lag-1))];
                for i = 0:hvar
                    PHItemp = J*A^i*J';
                    B(:,K*i+1:K*i+K) = PHItemp(1:K,1:K);
                end

                % Structural IRFs    
                % impact(1,ivaruy) = dsge_impact_uy./P(ivaruy,ivaruy);
                for i = 0:hvar
                    PHItemp = J*A^i*J';        
                    %Dimensions: (horizion,variables,shocks,reps,lags)
                    irfs(i+1,:,:,irep,ilag) = PHItemp*P.*repmat(impact,[K,1]);
                end
                
                irfs_rmse = squeeze(irfs);
                %   Model responses
                hs = 1:41;
                load(['mats/girf-' O.mod '.mat'])
                dsgeirf_tmp = squeeze(girf(hs+1,V.y,ievolidx,:));
                dsgeirf = 100*diff(log(dsgeirf_tmp),[],2);
                varirf = irfs_rmse(hs,ivary,ivaruy,ilag);
                bias = varirf-dsgeirf;
                rmses = sum(abs(bias))
                
                % Uncertainty shock FEVD
                iota = zeros(K,1);
                iota(ivaruy) = 1;
                for ihvar = 1:nhvars_fevd
                    hvar_fevd = hvars_fevd(ihvar);            
                    for ivar = 1:K
                        top = 0;
                        bottom = 0;
                        for j=0:hvar_fevd
                            Bvar = B(ivar,K*j+1:K*j+K);
                            top = top + (Bvar*P*iota)*(iota'*P'*Bvar');
                            bottom = bottom + Bvar*Sigma*Bvar';
                        end
                        fevds(ivar,ihvar,irep) = top/bottom;
                    end
                end  

                % Correlations of DSGE structural shocks and Cholesky identified shocks 
                struct_shocks_var = transpose(P\U(1:K,:));
                struct_shocks_dsge = shockslong((nburn+1+1+lag):(nburn+ndata),:,irep);
                %   e level shock v. VAR output shock
                corrarray(1,irep) = corr(struct_shocks_dsge(:,ielevidx), struct_shocks_var(:,ivary));
                %   e vol shock v. VAR output shock
                corrarray(2,irep) = corr(struct_shocks_dsge(:,ievolidx), struct_shocks_var(:,ivary));
                %   a level shock v. VAR output shock
                corrarray(3,irep) = corr(struct_shocks_dsge(:,ialevidx), struct_shocks_var(:,ivary));
                %   e level shock v. VAR uncertainty shock
                corrarray(4,irep) = corr(struct_shocks_dsge(:,ielevidx), struct_shocks_var(:,ivaruy));
                %   e vol shock v. VAR uncertainty shock
                corrarray(5,irep) = corr(struct_shocks_dsge(:,ievolidx), struct_shocks_var(:,ivaruy));
                 %  a level shock v. VAR uncertainty shock
                corrarray(6,irep) = corr(struct_shocks_dsge(:,ialevidx), struct_shocks_var(:,ivaruy));
            end

            % Shock correlations table
            corrarray_mean = mean(corrarray,2);
            tab = array2table(corrarray_mean);
            tab = varfun(@(x) num2str(x, ['%',sprintf('.%df',2)]), tab);
            tab.Properties.RowNames = {...
                'Corr(e lev shock, VAR y shock)',...
                'Corr(e vol shock, VAR y shock)',...
                'Corr(a lev shock, VAR y shock)',...
                'Corr(e lev shock, VAR U shock)',...
                'Corr(e vol shock, VAR U shock)',...
                'Corr(a lev shock, VAR U shock)'};
            tab.Properties.VariableNames = {'Mean'};
            %disp(tab)

            % FEVD table    
            fevd_mean = 100*squeeze(mean(fevds,3));
            tab = array2table(fevd_mean(ivary,:)');
            tab = varfun(@(x) num2str(x, ['%',sprintf('.%df',1)]), tab);
            tab.Properties.RowNames = colstrs;
            tab.Properties.VariableNames = {'Output FEVD'};
            %disp(tab) 
        end
        if saving==1
            save(['mats/var-' firstvar '-' middlevar '-' lastvar '-' mod '.mat'],...
                'irfs','varidx','lags','ievolidx')
        end
    end
end