% Simulate model and estimate VAR
clear; %clc; 
close all
warning('off','backtrace')

%Saving irf file
saving = 0;

% VAR options
O.mod = 'base';       % Baseline: base; Quasi-recursive: qr
Ovar.cholflag = 0;  % Cholesky = 1
K = 3;              % Number of variables
lags = 4;           % Set number of lags
nlags = numel(lags);

% VAR order
V = variables;
firstvar = 'uy';
secondvar = 'y';
if K == 3
    lastvarlist = {'i','a','c','n','w'};  %consumption: c; investment: i; TFP: a;
end

nlastvar = size(lastvarlist,2);

% Max share method (if applicable)
if Ovar.cholflag ==1
    Ovar.carrieroflag = NaN; 
elseif Ovar.cholflag == 0
    % Carriero and Volpicella = 1 
    % Standard max share = 0
    Ovar.carrieroflag = 0;
end
if Ovar.carrieroflag ==0
    Ovar.ntarget = 1; %Max-share on first variable
else
    Ovar.ntarget = 3; %Set 2 to K
end
if Ovar.ntarget>K
    disp('# of identified shocks cannot be > # of variables in VAR');
    return
end

% Set target horizons. Can be set different for K > 1
if Ovar.ntarget ==1
    Ovar.hvar = 80;
elseif Ovar.ntarget ==2
    Ovar.hvar = [0;0]; 
elseif Ovar.ntarget ==3
    Ovar.hvar = [80;80;80];
end
Ovar.hirf = 81;

% FEVD horizons
Ovar.hvars_fevd = [4,20,40,80];
Ovar.nhvars_fevd = numel(Ovar.hvars_fevd);
for ihvar=1:Ovar.nhvars_fevd
    colstrs_fevd{ihvar} = ['h=' num2str(Ovar.hvars_fevd(ihvar))];
end

% Load solution
fname = ['mats/Msolution-dens-' O.mod '.mat'];
load(fname);

% Load simulations
load(['mats/simdata-' O.mod '.mat'])
sims(sims(:,V.uy)<0,V.uy)=0.0001;
sims(isnan(sims(:,V.uy)),V.uy)=0.0001;

ievolidx = 1;
ielevidx = 2;
ialevidx = 3;

% VAR
disp('Estimating VAR...');
%   Dimension: Horizons, Variables, Shocks, Reps, Lags
irfs = NaN(Ovar.hirf+1,K,K,F.nreps,nlags);
%   Dimension: Shocks, Variables, Horizons, Reps
fevds = NaN(K,K,Ovar.nhvars_fevd,F.nreps);

for ivar = 1:nlastvar
    lastvar = lastvarlist{ivar};
    varidx = {firstvar,secondvar,lastvar};
    Ovar.ivaruy = find(strcmp(varidx,'uy'));
    Ovar.ivary = find(strcmp(varidx,'y'));
    Ovar.ivarlast = find(~strcmp(varidx,'uy') & ~strcmp(varidx,'y'));

    for ilag = 1:nlags
        lag = lags(ilag);
        Ovar.lag = lag;
        disp(['Order: ',varidx{1},', ',varidx{2},', ',varidx{3},', lag' num2str(lag)])    
        corrarray = zeros(K*3,F.nreps);
        for irep = 1:F.nreps 
            % Output
            simy = 100*log(sims(2:end,V.y,irep));
            % Uncertainty
            simuy = 100*log(sims(2:end,V.uy,irep));            
            if K ==3
                % Last variable
                if strcmp(lastvar,'c')
                    simlast = 100*log(sims(2:end,V.c,irep));
                elseif strcmp(lastvar,'i')
                    simlast = 100*log(sims(2:end,V.i,irep));
                elseif strcmp(lastvar,'a')
                    simlast = 100*sims(2:end,V.a,irep);
                elseif strcmp(lastvar,'w')
                    simlast = 100*log(sims(2:end,V.w,irep));
                elseif strcmp(lastvar,'n')
                    simlast = 100*log(sims(2:end,V.n,irep));
                end
            end

            % Assign data in VAR order
            if K == 2 
                vardata = [simuy,simy];
            elseif K == 3
                vardata = [simuy,simy,simlast];
            end

            % Estimate
            [irfs(:,:,:,irep,ilag),x,struct_shocks_var,fevds(:,:,:,irep)] = var_maxshare(vardata,F,V,Ovar);
            
            irfs_rmse = squeeze(irfs);
            %   Model responses
            hs = 1:41;
            load(['mats/girf-' O.mod '.mat'])
            dsgeirf_tmp = squeeze(girf(hs+1,V.y,ievolidx,:));
            dsgeirf = 100*diff(log(dsgeirf_tmp),[],2);
            varirf = irfs_rmse(hs,Ovar.ivary,Ovar.ivaruy,ilag);
            bias = varirf-dsgeirf;
            rmses = sum(abs(bias))
            
            % Correlations of DSGE structural shocks and identified shocks 
            struct_shocks_dsge = shockslong((F.nburn+1+1+lag):(F.nburn+F.ndata),:,irep);
            %   e level shock v. VAR uncertainty shock
            corrarray(1,irep) = corr(struct_shocks_dsge(:,ielevidx), struct_shocks_var(:,Ovar.ivaruy));
            %   e vol shocl v. VAR uncertainty shock
            corrarray(2,irep) = corr(struct_shocks_dsge(:,ievolidx), struct_shocks_var(:,Ovar.ivaruy));
            %  a level shock v. VAR uncertainty shock
            corrarray(3,irep) = corr(struct_shocks_dsge(:,ialevidx), struct_shocks_var(:,Ovar.ivaruy));
            %   e level shock v. VAR output shock
            corrarray(4,irep) = corr(struct_shocks_dsge(:,ielevidx), struct_shocks_var(:,Ovar.ivary));
            %   e vol shock v. VAR output shock
            corrarray(5,irep) = corr(struct_shocks_dsge(:,ievolidx), struct_shocks_var(:,Ovar.ivary));
            %   a level shock v. VAR output shock
            corrarray(6,irep) = corr(struct_shocks_dsge(:,ialevidx), struct_shocks_var(:,Ovar.ivary));
            if K ==3
                %   e level shock v. VAR third shock
                corrarray(7,irep) = corr(struct_shocks_dsge(:,ielevidx), struct_shocks_var(:,Ovar.ivarlast));
                %   e vol shock v. VAR third shock
                corrarray(8,irep) = corr(struct_shocks_dsge(:,ievolidx), struct_shocks_var(:,Ovar.ivarlast));
                %   a level shock v. VAR third shock
                corrarray(9,irep) = corr(struct_shocks_dsge(:,ialevidx), struct_shocks_var(:,Ovar.ivarlast));
            end
        end      

        % Shock correlations table
        corrarray_mean = mean(corrarray,2);
        tab = array2table(corrarray_mean);
        tab = varfun(@(x) num2str(x, ['%',sprintf('.%df',2)]), tab);
        if K ==2 
            tab.Properties.RowNames = {...
            'Corr(e lev shock, VAR U shock)',...
            'Corr(e vol shock, VAR U shock)',...
            'Corr(a lev shock, VAR U shock)',...
            'Corr(e lev shock, VAR y shock)',...
            'Corr(e vol shock, VAR y shock)',...
            'Corr(a lev shock, VAR y shock)'};
        elseif K ==3
             tab.Properties.RowNames = {...
            'Corr(e lev shock, VAR U shock)',...
            'Corr(e vol shock, VAR U shock)',...
            'Corr(a lev shock, VAR U shock)',...
            'Corr(e lev shock, VAR y shock)',...
            'Corr(e vol shock, VAR y shock)',...
            'Corr(a lev shock, VAR y shock)',...
            'Corr(e lev shock, VAR last shock)',...
            'Corr(e vol shock, VAR last shock)',...
            'Corr(a lev shock, VAR last shock)'};
        end
        tab.Properties.VariableNames = {'Mean'};
        %disp(tab)

        % FEVD table 
        % fevd_mean = 100*squeeze(mean(fevds,4));
        % tab = array2table(fevd_mean(ivary,:)');
        % tab = varfun(@(x) num2str(x, ['%',sprintf('.%df',1)]), tab);
        % tab.Properties.RowNames = colstrs;
        % tab.Properties.VariableNames = {'Output FEVD'};
        % disp(tab)     
    end
    if saving==1
        if Ovar.cholflag
            if K ==2
                filename = ['mats/var-' firstvar '-' secondvar '-' O.mod '-chol.mat'];
            elseif K ==3
                filename = ['mats/var-' firstvar '-' secondvar '-' lastvar '-' O.mod '-chol.mat'];
            end
        else
            if K ==2
                filename = ['mats/var-' firstvar '-' secondvar '-' O.mod '-ntarget' num2str(Ovar.ntarget) ...
                    '-hvar' num2str(Ovar.hvar(1)) '-carr' num2str(Ovar.carrieroflag) '.mat'];
            elseif K ==3
                filename = ['mats/var-' firstvar '-' secondvar '-' lastvar '-' O.mod '-ntarget' num2str(Ovar.ntarget) ...
                    '-hvar' num2str(Ovar.hvar(1)) '-carr' num2str(Ovar.carrieroflag) '.mat'];
            end
        end
        save(filename,'irfs','varidx','lags','ievolidx')
    end
end