% Compute RMSEs and plot IRFs
if exist('genresults','var')
    saving = 0;
    plotting = 0;
else
    clear; clc; close all; 
    mod = 'qr';
    middlevar = 'i'; 
    saving = 0;
    plotting = 1;  
    plotverson = ''; 
end

%% Load mats
%   VAR responses
firstvars = {'uy','y'};
lastvars = flip(firstvars);
if strcmp(mod,'qr')
    firstvars = firstvars(1);
    lastvars = lastvars(1);
end
norders = numel(firstvars);
for iorder = 1:norders
    firstvar = firstvars{iorder};
    lastvar = lastvars{iorder};
    load(['mats/var-' firstvar '-' middlevar '-' lastvar '-' mod '.mat'])
    irfs_all{iorder} = squeeze(irfs);
    varidx_all{iorder} = varidx;
end
%   Model responses
load(['mats/girf-' mod '.mat'])

%% Root mean squared error
hs = 1:41; ilag = (lags == 4); 
dsgeirf_tmp = squeeze(girf(hs+1,V.y,ievolidx,:));
dsgeirf = 100*diff(log(dsgeirf_tmp),[],2);
rmses = NaN(1,norders);
for iorder = 1:norders
    iy = find(strcmp(varidx_all{iorder},'y'));
    iuy = find(strcmp(varidx_all{iorder},'uy'));
    varirf = irfs_all{iorder}(hs,iy,iuy,ilag);
    bias = varirf-dsgeirf;
    rmses(iorder) = sum(abs(bias));
end

%% Plot impulse responses
if plotting == 1
    plotops; nplot = 41;
    figure
    fontsize = 10;
    ilag = (lags == 4);
    xaxis = (1:nplot)-1;
    legstrs = {'Cholesky: $\mathcal{U}$ First','Cholesky: $\mathcal{U}$ Last','Population'};
    figpos = [1,5,3.2,3.1];
    axispos = [.1,.07,.86,.86];
    if strcmp(plotverson,'P')
        axispos = [.12,.1,.82,.82];
    end
    plottype = {'r--','b-.','k-'};
    if strcmp(mod,'qr')
        legstrs = legstrs([1,3]);
        plottype = plottype([1,3]);
    end
    set(gcf,'Position',figpos,...
        'PaperPosition',[0,0,figpos(3:4)],...
        'PaperSize',figpos(3:4),...
        'name',mod);
    set(gca,'Position',axispos);
    hold on; box on; grid on;
    % VAR responses
    for iorder = 1:norders
        iy = find(strcmp(varidx_all{iorder},'y'));
        iuy = find(strcmp(varidx_all{iorder},'uy'));
        yaxis = squeeze(irfs_all{iorder}(1:nplot,iy,iuy,ilag)); 
        plot(xaxis,yaxis,plottype{iorder}); 
    end
    % Model responses
    plotdata = squeeze(girf(2:nplot+1,V.y,ievolidx,:));
    yaxis = 100*diff(log(plotdata),[],2);
    plot(xaxis,yaxis,plottype{norders+1});
    ylim([-0.9,0.1]); yticks(-0.9:0.1:0.1)
    % Zero line
    yline(0,'LineWidth',1);
    % Title
    title('Model 1: $\varepsilon_e,\varepsilon_a,\varepsilon_{ev}$')
    % Legend
    legend(legstrs,'Location','SouthEast','FontSize',fontsize)
    if strcmp(plotverson,'P')
        title('Output ($\%$)','interpreter','latex','FontSize',fontsize)
    end
    set(findall(gcf,'-property','FontSize'),'FontSize',fontsize)
    set(findall(gcf,'-property','interpreter'),'interpreter','latex')
    % Save
    if saving==1
        savename = ['responses-' mod];
        if strcmp(plotverson,'P')
            savename = [savename '-' plotverson];
        end
        saveas(gcf,['results/' savename '.pdf'])
    end

    % Compare alternative lags
    % xaxis = (1:nplot)-1;
    % figpos = [1,5,3.2,3.2];
    % axispos = [.09,.07,.86,.90];
    % fontsize = 8;
    % % VAR responses
    % for ilag = 1:numel(lags)
    %     legstrs{ilag} = ['$p = ',num2str(lags(ilag)),'$'];
    % end
    % for iorder = 1:norders
    %     figure
    %     plottype = {'r--','b-.','k-'};
    %     set(gcf,'Position',figpos,...
    %         'PaperPosition',[0,0,figpos(3:4)],...
    %         'PaperSize',figpos(3:4),...
    %         'name',['first var = ' firstvars{iorder}]);
    %     set(gca,'Position',axispos);
    %     hold on; box on; grid on;
    %     for ilag = 1:numel(lags)
    %         iy = find(strcmp(varidx_all{iorder},'y'));
    %         iuy = find(strcmp(varidx_all{iorder},'uy'));
    %         yaxis = squeeze(irfs_all{iorder}(1:nplot,iy,iuy,ilag)); 
    %         plot(xaxis,yaxis,plottype{ilag}); 
    %     end
    %     % Legend
    %     legend(legstrs,'Location','NorthWest','FontSize',fontsize)
    %     set(findall(gcf,'-property','FontSize'),'FontSize',fontsize)
    %     set(findall(gcf,'-property','interpreter'),'interpreter','latex')
    % end
end