% Compute RMSEs and plot IRFs
if exist('genresults','var')
    saving = 0;
    plotting = 0;    
else
    clear; clc; close all; 
    mod = 'base';
    saving = 0;
    plotting = 1;    
    plotverson = '';
    varslist = {'uy','y','n'};
end

% Cases flags
hvarflags = [0 0];
carrieroflags = [0 1];  %[maxshare;carriero]
ntargets = [1 3];       %Number of targets

%% Load mats
%   VAR responses

norders = numel(carrieroflags);
for iorder = 1:norders
    load(['mats/var-' varslist{1} '-' varslist{2} '-' varslist{3} '-' mod ...
        '-ntarget' num2str(ntargets(iorder)) '-hvar' num2str(hvarflags(iorder)) ...
        '-carr' num2str(carrieroflags(iorder)) '.mat']);
    irfs_all{iorder} = squeeze(irfs);
    varidx_all{iorder} = varidx;
end
%   Model responses
load(['mats/girf-' mod '.mat'])

%% Root mean squared error
hs = 1:41; ilag = (lags == 4);
dsgeirf_tmp = squeeze(girf(hs+1,V.y,ievolidx,:));
dsgeirf = 100*diff(log(dsgeirf_tmp),[],2);
rmses = NaN(1,norders);
for iorder = 1:norders
    iy = find(strcmp(varidx_all{iorder},'y'));
    iuy = find(strcmp(varidx_all{iorder},'uy'));
    varirf = irfs_all{iorder}(hs,iy,iuy,ilag);
    bias = varirf-dsgeirf;
    rmses(iorder) = sum(abs(bias));
end

%% Plot impulse responses
if plotting == 1
    plotops; nplot = 41;
    figure
    fontsize = 8;
    ilag = (lags == 4);
    xaxis = (1:nplot)-1;
    legstrs = {'Max share','Carriero','Population'};
    % figpos = [1,5,2.1,2.1];
    % axispos = [.12,.1,.82,.85];
    figpos = [1,5,3.2,3.2];
    axispos = [.09,.07,.86,.90];
    if strcmp(plotverson,'P')
        axispos = [.12,.1,.82,.82];
    end
    plottype = {'r--','b-.','k-'};
    set(gcf,'Position',figpos,...
        'PaperPosition',[0,0,figpos(3:4)],...
        'PaperSize',figpos(3:4),...
        'name',mod);
    set(gca,'Position',axispos);
    hold on; box on; grid on;
    % VAR responses
    for iorder = 1:norders
        iy = find(strcmp(varidx_all{iorder},'y'));
        iuy = find(strcmp(varidx_all{iorder},'uy'));
        yaxis = squeeze(irfs_all{iorder}(1:nplot,iy,iuy,ilag)); 
        plot(xaxis,yaxis,plottype{iorder}); 
    end
    % Model responses
    plotdata = squeeze(girf(2:nplot+1,V.y,ievolidx,:));
    yaxis = 100*diff(log(plotdata),[],2);
    plot(xaxis,yaxis,plottype{norders+1});
    %ylim([-0.7,0.1]); yticks(-0.7:0.1:0.1)
    % Legend
    legend(legstrs,'Location','SouthEast','FontSize',fontsize)
    if strcmp(plotverson,'P')
        title('Output ($\%$)','interpreter','latex','FontSize',fontsize)
    end
    set(findall(gcf,'-property','FontSize'),'FontSize',fontsize)
    set(findall(gcf,'-property','interpreter'),'interpreter','latex')
    % Save
    if saving==1
        savename = ['responses-' mod];
        if strcmp(plotverson,'P')
            savename = [savename '-' plotverson];
        end
        saveas(gcf,['results/' savename '.pdf'])
    end
end