%Plot IRFs
clear; clc; close all; 
saving = 0;

pres_flag = 0; %Individual IRFs saved
pres_saving = 0;

% Option
mod = 'base';

%% Load mats
%  Models
folders = {'epse-epsev-epsa','epse-epsav-epsa'};
nfolders = numel(folders);
%  Orderings
firstvars = {'uy','y'};
middlevar = 'i'; 
lastvars = flip(firstvars);
if strcmp(mod,'qr')
    firstvars = firstvars(1);
    lastvars = lastvars(1);
end
norders = numel(firstvars);
for ifolder = 1:nfolders
    folder = folders{ifolder};
    %   Model responses
    load([folder '/mats/girf-' mod '.mat'])
    girfs_all{ifolder} = girf;
    Vs{ifolder} = V;
    %   VAR responses
    for iorder = 1:norders
        firstvar = firstvars{iorder};
        lastvar = lastvars{iorder};
        load([folder '/mats/var-' firstvar '-' middlevar '-' lastvar '-' mod '.mat'])
        irfs_all{iorder,ifolder} = squeeze(irfs);
        varidx_all{iorder,ifolder} = varidx;
        if ifolder==1
            volidx(ifolder) = ievolidx;
        elseif ifolder==2
            volidx(ifolder) = iavolidx;
        end
    end
end

%% Plot impulse responses

% Plot options
plotops
nplot = 41;
figpos = [1,5,6.5,3];
pad.topmarg = .07;
pad.leftmarg = .05;
pad.vspace = .08;
pad.hspace = .08;
pad.rightmarg = .02;
pad.botmarg = .08;
ilag = (lags == 4);
xaxis = (1:nplot)-1;
fontsize = 10;
titles = {'Model 1: $\varepsilon_e,\varepsilon_a,\varepsilon_{ev}$',...
          'Model 2: $\varepsilon_e,\varepsilon_a,\varepsilon_{av}$'};
plotdim = [1,nfolders];
plottype = {'r--','b-.','k-'};
legstrs = {'Cholesky: $\mathcal{U}$ First','Cholesky: $\mathcal{U}$ Last','Population'};
if strcmp(mod,'qr')
    legstrs = legstrs([1,3]);
    plottype = plottype([1,3]);
end

% Execute plot
figure
set(gcf,'Position',figpos,...
    'PaperPosition',[0 0 figpos(3:4)],...
    'PaperSize',figpos(3:4),...
    'Name',mod);
for ifolder = 1:nfolders
    subplot(plotdim(1),plotdim(2),ifolder); hold on; box on; grid on;
    % VAR responses
    for iorder = 1:norders
        iy = find(strcmp(varidx_all{iorder,ifolder},'y'));
        iuy = find(strcmp(varidx_all{iorder,ifolder},'uy'));
        yaxis = squeeze(irfs_all{iorder,ifolder}(1:nplot,iy,iuy,ilag)); 
        plot(xaxis,yaxis,plottype{iorder}); 
    end
    % Model responses
    plotdata = squeeze(girfs_all{ifolder}(2:nplot+1,Vs{ifolder}.y,volidx(ifolder),:));
    yaxis = 100*diff(log(plotdata),[],2);
    plot(xaxis,yaxis,plottype{norders+1});
    ylim([-0.9,0.1]); yticks(-0.9:0.1:0.1)
    % Zero line
    yline(0,'LineWidth',1);
    % Title
    title(titles{ifolder})
    % Legend
    if ifolder==1
        legend(legstrs,'Location','SouthEast','FontSize',fontsize)  
    end
end
subplotpad(gcf,pad); 
set(findall(gcf,'-property','FontSize'),'FontSize',fontsize)
set(findall(gcf,'-property','interpreter'),'interpreter','latex')
% Save
if saving==1
    saveas(gcf,['responses-' mod '.pdf'])
end

if pres_flag
    figpos = [1,5,3,3];
    for ifolder = 1:nfolders
        figure
        set(gcf,'Position',figpos,...
        'PaperPosition',[0 0 figpos(3:4)],...
        'PaperSize',figpos(3:4),...
        'Name',mod);
        %subplot(plotdim(1),plotdim(2),ifolder);
        % VAR responses
        for iorder = 1:norders
            iy = find(strcmp(varidx_all{iorder,ifolder},'y'));
            iuy = find(strcmp(varidx_all{iorder,ifolder},'uy'));
            yaxis = squeeze(irfs_all{iorder,ifolder}(1:nplot,iy,iuy,ilag)); 
            plot(xaxis,yaxis,plottype{iorder}); hold on; box on; grid on;
        end
        % Model responses
        plotdata = squeeze(girfs_all{ifolder}(2:nplot+1,Vs{ifolder}.y,volidx(ifolder),:));
        yaxis = 100*diff(log(plotdata),[],2);
        plot(xaxis,yaxis,plottype{norders+1});
        ylim([-0.9,0.1]); yticks(-0.9:0.1:0.1)
        % Zero line
        yline(0,'LineWidth',1);
        % Title
        title('Response of output to uncertainty shock')
        %title(titles{ifolder})
        % Legend
        %if ifolder==1
            legend(legstrs,'Location','SouthEast','FontSize',fontsize)  
        %end
        set(findall(gcf,'-property','FontSize'),'FontSize',fontsize)
        set(findall(gcf,'-property','interpreter'),'interpreter','latex')
        % Save
        if pres_saving==1
            saveas(gcf,['responses-pres-' num2str(ifolder) '-' mod '.pdf'])
        end
    end
    %subplotpad(gcf,pad); 
end